
;**************************************************************************
;*                        Watch Dog - 5V                                  *
;**************************************************************************
;**************************************************************************
;*                                                                        *
;*          v 1.05 - 18.01.2017                                           *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC16F1823 used                                       *
;*          OSC.......: Internal osc. used     POWER.....: 5,0V accu.     *
;**************************************************************************


 INCLUDE "P16F1823.INC"


_U001			     EQU     B'00100110000100'		;see Page 48 of Datasheet
_U002			     EQU     B'01100011111111'		;see Page 49 of Datasheet


	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002


;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"
TIMER3		EQU	H'22'		;delay routine

v_input		EQU	H'23'		;in bit 3 is store last value of the input (RA3)
v_current	EQU	H'24'		;current value of RA3 is stored in bit3

v_wdt		EQU	H'25'		;count down timer
v_counter	EQU	H'26'		;main watch dog counter
v_cnt_dsp	EQU	H'27'
v_failures	EQU	H'28'		;number of restarts
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------


;--------------------------------------------------------------------------
;--------------------------------------------------------------------------


;==========================================================================

		ORG	0		;Reset vector address
		goto	start

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
start		nop

		clrf	BSR
		movlw	H'FF'
		movwf	PORTC
;
		clrf	INTCON		;disable interrupts

		banksel	OPTION_REG	;BANK 1
		movlw	B'01111111'
		movwf	OPTION_REG	;see datasheet Page 176

		movlw	H'03'
		movwf	BSR		;select BANK 3
		clrf	ANSELA
		clrf	ANSELC

		movlw	H'01'
		movwf	BSR		;select BANK 1
		movlw	B'11011000'
		movwf	TRISA

		movlw	B'11000000'
		movwf	TRISC

		movf	OSCCON,0
		andlw	B'11101111'
		iorlw	B'01101000'
		movwf	OSCCON		;set 4MHz internal clock (default after reset is 500kHz)

		movlw	H'04'
		movwf	BSR		;select BANK 4
		movlw	B'00011000'
		movwf	WPUA		;enable weak pull-ups for RA3 and RA4
		clrf	WPUC		;disable weak pull-ups for PORTC

		banksel	LATC
		clrf	LATC

		clrf	BSR		;select BANK 0
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
start_up	nop

;switch off everything ------------------
		movlw	H'FF'
		movwf	PORTC
		movlw	B'00100000'
		movwf	PORTA
;----------------------------------------
rst_01		call	power_off
		call	wm_9
		call	dr6
		call	wm_8
		call	dr6
		call	wm_7
		call	dr6
		call	wm_6
		call	dr6
		call	wm_5
		call	dr6
		call	wm_4
		call	dr6
		call	wm_3
		call	dr6
		call	wm_2
		call	dr6
		call	wm_1
		call	dr6
		call	wm_0
		call	dr6
;--------------------------------------
		call	wm_minus
		call	power_on
		clrf	v_input
		clrf	v_wdt
		clrf	v_cnt_dsp

		movlw	D'010'
		movwf	v_counter

		call	EEPROM_read
;--------------------------------------
;--------------------------------------
;main loop
;--------------------------------------
;--------------------------------------
main_loop	call	point_on
		call	w_display1
		call	dr6_wdt
		call	point_off
; test the button ------------------------
		btfss	PORTA,D'004'
		call	button_pressed
; ----------------------------------------
		call	w_display2
		call	dr6_wdt

		incf	v_wdt,1
		movf	v_wdt,0
		xorlw	D'005'
		btfsc	STATUS,Z
		call	dec_counter

		goto	main_loop
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
button_pressed	call	wm_o
		call	d_1sec
		call	d_1sec
		call	d_1sec
		btfsc	PORTA,D'004'
		return			;button has been released
		call	wm_E
		clrf	v_failures
		call	EEPROM_write
		call	d_1sec
		return
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
got_it		clrf	v_wdt
		movf	v_current,0
		movwf	v_input
		movlw	D'010'
		movwf	v_counter
		clrf	v_cnt_dsp
		return
;---------------------------------------------------------------------------------------
dec_counter	clrf	v_wdt
		decf	v_counter,1
		bsf	v_cnt_dsp,D'000'

		movf	v_counter,0
		btfss	STATUS,Z
		return
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;Shut down required
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;count down finished -------------------------------------------------------------------
		call	power_off

		incf	v_failures,1
		movf	v_failures,0
		xorlw	D'011'
		btfss	STATUS,Z
		call	EEPROM_write

		call	off_wait_per
		goto	rst_01			;re-start
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
off_wait_per	call	off_blink
		call	off_blink
		call	off_blink
		call	off_blink
		call	off_blink
		call	off_blink
		return
;---------------------------------------------------------------------------------------
off_blink	call	wm_space
		call	d_1sec
		call	wm_0
		call	d_1sec
		return
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
w_display1	btfss	v_cnt_dsp,D'000'
		goto	wm_failures
		movf	v_counter,0
		brw
		goto	wm_0
		goto	wm_1
		goto	wm_2
		goto	wm_3
		goto	wm_4
		goto	wm_5
		goto	wm_6
		goto	wm_7
		goto	wm_8
		goto	wm_9
;---------------------------------------------------------------------------------------
w_display2	btfss	v_cnt_dsp,D'000'
		goto	wm_failures
		goto	wm_space
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
wm_failures	movf	v_failures,0
		brw
		goto	wm_minus
		goto	wm_1
		goto	wm_2
		goto	wm_3
		goto	wm_4
		goto	wm_5
		goto	wm_6
		goto	wm_7
		goto	wm_8
		goto	wm_9
		goto	wm_more			;more than 9 failures deteceted
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;Character set
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
wm_minus	movlw	B'00111111'
		movwf	PORTC
		bcf	PORTA,D'005'
		return
;--------------------------------------------
wm_space	movlw	B'00111111'
		movwf	PORTC
		bsf	PORTA,D'005'
		return
;--------------------------------------------
wm_o		movlw	B'00100011'
		movwf	PORTC
		bsf	PORTA,D'005'
		return
;--------------------------------------------
wm_E		movlw	B'00000110'
		movwf	PORTC
		bcf	PORTA,D'005'
		return
;--------------------------------------------
wm_0		movlw	B'00000000'
		movwf	PORTC
		bsf	PORTA,D'005'
		return
;--------------------------------------------
wm_1		movlw	B'00111001'
		movwf	PORTC
		bsf	PORTA,D'005'
		return
;--------------------------------------------
wm_2		movlw	B'00100100'
		movwf	PORTC
		bcf	PORTA,D'005'
		return
;--------------------------------------------
wm_3		movlw	B'00110000'
		movwf	PORTC
		bcf	PORTA,D'005'
		return
;--------------------------------------------
wm_4		movlw	B'00011001'
		movwf	PORTC
		bcf	PORTA,D'005'
		return
;--------------------------------------------
wm_5		movlw	B'00010010'
		movwf	PORTC
		bcf	PORTA,D'005'
		return
;--------------------------------------------
wm_6		movlw	B'00000010'
		movwf	PORTC
		bcf	PORTA,D'005'
		return
;--------------------------------------------
wm_7		movlw	B'00011000'
		movwf	PORTC
		bsf	PORTA,D'005'
		return
;--------------------------------------------
wm_8		movlw	B'00000000'
		movwf	PORTC
		bcf	PORTA,D'005'
		return
;--------------------------------------------
wm_9		movlw	B'00010000'
		movwf	PORTC
		bcf	PORTA,D'005'
		return
;--------------------------------------------
wm_more		movlw	B'00110110'
		movwf	PORTC
		bcf	PORTA,D'005'
		return
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;--------------------------------------------
point_on	nop
		bcf	PORTA,D'004'
		banksel	TRISA
		bcf	TRISA,D'004'
		clrf	BSR
		return
;--------------------------------------------
point_off	nop
		bcf	PORTA,D'004'
		banksel	TRISA
		bsf	TRISA,D'004'
		clrf	BSR
		return
;--------------------------------------------
green_on	bsf	PORTA,D'002'
		bcf	PORTA,D'001'
		return
;----------------------------------------
red_on		bsf	PORTA,D'001'
		bcf	PORTA,D'002'
		return
;---------------------------------------------------------------------------------------
power_off	bcf	PORTA,D'000'
		call	red_on
		return
;---------------------------------------------------------------------------------------
power_on	bsf	PORTA,D'000'
		call	green_on
		return
;---------------------------------------------------------------------------------------
EEPROM_read	nop
		banksel	EEADRL
		movlw	H'00'
		movwf	EEADRL			;EEPROM Address = 00h
		bcf	EECON1, CFGS		;deselect config space
		bcf	EECON1, EEPGD		;point to data memory
		bsf	EECON1, RD		;EEPROM read
		movf	EEDATL,0		;read the data
		clrf	BSR			;select Bank 0
		movwf	v_failures
		movf	v_failures,0
		xorlw	H'FF'
		btfss	STATUS,Z
		return
		clrf	v_failures		;in case there was no write to EEPROM at all
		return
;---------------------------------------------------------------------------------------
EEPROM_write	nop
		movf	v_failures,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'00'
		movwf	EEADRL
		bcf	EECON1, CFGS		;deselect config space
		bcf	EECON1, EEPGD		;point to data memory
		bsf	EECON1, WREN		;EEPROM read
		movlw	H'55'
		movwf	EECON2			;write 55h
		movlw	H'AA'
		movwf	EECON2			;write AAh
;
		bsf	EECON1, WR		;start write operation
		bcf	EECON1, WREN		;disable writes
wr_wait		btfsc	EECON1, WR		;wait until write has been completed
		goto	wr_wait

		clrf	BSR
		return
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;		*********************************************
;		*               delay routine               *
;		*********************************************

;------------------------------------
d2_wdt		movlw	D'255'
		movwf	TIMER2
DEL_LOOP1g	movlw	D'255'
		movwf	TIMER1

		movf	PORTA,0
		andlw	B'00001000'
		movwf	v_current
		xorwf	v_input,0
		btfss	STATUS,Z
		call	got_it

DEL_LOOP2g	decfsz	TIMER1,F
		goto	DEL_LOOP2g
		decfsz	TIMER2,F
		goto	DEL_LOOP1g
		return
;------------------------------------
dr6_wdt		call	d2_wdt
		call	d2_wdt
		return
;------------------------------------


;------------------------------------
d2		movlw	D'255'
		movwf	TIMER2
DEL_LOOP1	movlw	D'255'
		movwf	TIMER1

DEL_LOOP2	decfsz	TIMER1,F
		goto	DEL_LOOP2
		decfsz	TIMER2,F
		goto	DEL_LOOP1
		return
;------------------------------------
dr2		call	d2
		call	d2
		return
;------------------------------------
dr6		call	dr2
		call	dr2
		return
;------------------------------------
dr10		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		return
;------------------------------------
d1		movlw	D'100'
		movwf	TIMER2
DEL_LOOP11	movlw	D'255'
		movwf	TIMER1

DEL_LOOP12	decfsz	TIMER1,F
		goto	DEL_LOOP12
		decfsz	TIMER2,F
		goto	DEL_LOOP11
		return
;------------------------------------
d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return

;------------------------------------
;------------------------------------
;tuning:
;	005 x 255 x 255 = 325.125 -- a little bit too fast
;	006 x 218 x 250 = 327.000 -- still too fast
;	006 x 219 x 250 = 328.500 -- still too fast
;	006 x 221 x 250 = 331.500 -- very little too slow; but for the application ok...
;------------------------------------
d_1sec		movlw	D'006'
		movwf	TIMER3
DEL_LOOPx0	movlw	D'221'
		movwf	TIMER2
DEL_LOOPx1	movlw	D'250'
		movwf	TIMER1
DEL_LOOPx2	decfsz	TIMER1,F
		goto	DEL_LOOPx2
		decfsz	TIMER2,F
		goto	DEL_LOOPx1
		decfsz	TIMER3,F
		goto	DEL_LOOPx0
		return
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

		END